@@ -3,10 +3,14 @@ from textual.widgets import Static
 
 
 class DisplayApp(App):
+    CSS_PATH = "display.tcss"
+
     def compose(self):
         yield Static("Widget 1")
         yield Static("Widget 2", classes="remove")
         yield Static("Widget 3")
 
 
-app = DisplayApp(css_path="display.css")
+if __name__ == "__main__":
+    app = DisplayApp()
+    app.run()
