@@ -3,7 +3,7 @@ from textual.widgets import Footer, Header, SelectionList
 
 
 class SelectionListApp(App[None]):
-    CSS_PATH = "selection_list.css"
+    CSS_PATH = "selection_list.tcss"
 
     def compose(self) -> ComposeResult:
         yield Header()
