@@ -4,7 +4,7 @@ import pytest
 
 from textual.color import Color
 from textual.css.errors import UnresolvedVariableError
-from textual.css.parse import substitute_references
+from textual.css.parse import is_id_selector, substitute_references
 from textual.css.scalar import Scalar, Unit
 from textual.css.stylesheet import Stylesheet, StylesheetParseError
 from textual.css.tokenize import tokenize
@@ -17,12 +17,12 @@ from textual.layouts.vertical import VerticalLayout
 class TestVariableReferenceSubstitution:
     def test_simple_reference(self):
         css = "$x: 1; #some-widget{border: $x;}"
-        variables = substitute_references(tokenize(css, ""))
+        variables = substitute_references(tokenize(css, ("", "")))
         assert list(variables) == [
             Token(
                 name="variable_name",
                 value="$x:",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 0),
                 referenced_by=None,
@@ -30,7 +30,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 3),
                 referenced_by=None,
@@ -38,7 +38,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="number",
                 value="1",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 4),
                 referenced_by=None,
@@ -46,7 +46,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="variable_value_end",
                 value=";",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 5),
                 referenced_by=None,
@@ -54,7 +54,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 6),
                 referenced_by=None,
@@ -62,7 +62,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="selector_start_id",
                 value="#some-widget",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 7),
                 referenced_by=None,
@@ -70,7 +70,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="declaration_set_start",
                 value="{",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 19),
                 referenced_by=None,
@@ -78,7 +78,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="declaration_name",
                 value="border:",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 20),
                 referenced_by=None,
@@ -86,7 +86,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 27),
                 referenced_by=None,
@@ -94,7 +94,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="number",
                 value="1",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 4),
                 referenced_by=ReferencedBy(
@@ -104,7 +104,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="declaration_end",
                 value=";",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 30),
                 referenced_by=None,
@@ -112,7 +112,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="declaration_set_end",
                 value="}",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 31),
                 referenced_by=None,
@@ -121,12 +121,12 @@ class TestVariableReferenceSubstitution:
 
     def test_simple_reference_no_whitespace(self):
         css = "$x:1; #some-widget{border: $x;}"
-        variables = substitute_references(tokenize(css, ""))
+        variables = substitute_references(tokenize(css, ("", "")))
         assert list(variables) == [
             Token(
                 name="variable_name",
                 value="$x:",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 0),
                 referenced_by=None,
@@ -134,7 +134,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="number",
                 value="1",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 3),
                 referenced_by=None,
@@ -142,7 +142,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="variable_value_end",
                 value=";",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 4),
                 referenced_by=None,
@@ -150,7 +150,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 5),
                 referenced_by=None,
@@ -158,7 +158,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="selector_start_id",
                 value="#some-widget",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 6),
                 referenced_by=None,
@@ -166,7 +166,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="declaration_set_start",
                 value="{",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 18),
                 referenced_by=None,
@@ -174,7 +174,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="declaration_name",
                 value="border:",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 19),
                 referenced_by=None,
@@ -182,7 +182,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 26),
                 referenced_by=None,
@@ -190,7 +190,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="number",
                 value="1",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 3),
                 referenced_by=ReferencedBy(
@@ -200,7 +200,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="declaration_end",
                 value=";",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 29),
                 referenced_by=None,
@@ -208,7 +208,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="declaration_set_end",
                 value="}",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 30),
                 referenced_by=None,
@@ -218,15 +218,31 @@ class TestVariableReferenceSubstitution:
     def test_undefined_variable(self):
         css = ".thing { border: $not-defined; }"
         with pytest.raises(UnresolvedVariableError):
-            list(substitute_references(tokenize(css, "")))
+            list(substitute_references(tokenize(css, ("", ""))))
+
+    def test_empty_variable(self):
+        css = "$x:\n* { background:$x; }"
+        result = list(substitute_references(tokenize(css, ("", ""))))
+        assert [(t.name, t.value) for t in result] == [
+            ("variable_name", "$x:"),
+            ("variable_value_end", "\n"),
+            ("selector_start_universal", "*"),
+            ("whitespace", " "),
+            ("declaration_set_start", "{"),
+            ("whitespace", " "),
+            ("declaration_name", "background:"),
+            ("declaration_end", ";"),
+            ("whitespace", " "),
+            ("declaration_set_end", "}"),
+        ]
 
     def test_transitive_reference(self):
         css = "$x: 1\n$y: $x\n.thing { border: $y }"
-        assert list(substitute_references(tokenize(css, ""))) == [
+        assert list(substitute_references(tokenize(css, ("", "")))) == [
             Token(
                 name="variable_name",
                 value="$x:",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 0),
                 referenced_by=None,
@@ -234,7 +250,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 3),
                 referenced_by=None,
@@ -242,7 +258,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="number",
                 value="1",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 4),
                 referenced_by=None,
@@ -250,7 +266,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="variable_value_end",
                 value="\n",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 5),
                 referenced_by=None,
@@ -258,7 +274,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="variable_name",
                 value="$y:",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 0),
                 referenced_by=None,
@@ -266,7 +282,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 3),
                 referenced_by=None,
@@ -274,7 +290,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="number",
                 value="1",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 4),
                 referenced_by=ReferencedBy(
@@ -284,7 +300,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="variable_value_end",
                 value="\n",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 6),
                 referenced_by=None,
@@ -292,7 +308,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="selector_start_class",
                 value=".thing",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(2, 0),
                 referenced_by=None,
@@ -300,7 +316,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(2, 6),
                 referenced_by=None,
@@ -308,7 +324,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="declaration_set_start",
                 value="{",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(2, 7),
                 referenced_by=None,
@@ -316,7 +332,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(2, 8),
                 referenced_by=None,
@@ -324,7 +340,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="declaration_name",
                 value="border:",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(2, 9),
                 referenced_by=None,
@@ -332,7 +348,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(2, 16),
                 referenced_by=None,
@@ -340,7 +356,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="number",
                 value="1",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 4),
                 referenced_by=ReferencedBy(
@@ -350,7 +366,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(2, 19),
                 referenced_by=None,
@@ -358,7 +374,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="declaration_set_end",
                 value="}",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(2, 20),
                 referenced_by=None,
@@ -367,11 +383,11 @@ class TestVariableReferenceSubstitution:
 
     def test_multi_value_variable(self):
         css = "$x: 2 4\n$y: 6 $x 2\n.thing { border: $y }"
-        assert list(substitute_references(tokenize(css, ""))) == [
+        assert list(substitute_references(tokenize(css, ("", "")))) == [
             Token(
                 name="variable_name",
                 value="$x:",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 0),
                 referenced_by=None,
@@ -379,7 +395,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 3),
                 referenced_by=None,
@@ -387,7 +403,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="number",
                 value="2",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 4),
                 referenced_by=None,
@@ -395,7 +411,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 5),
                 referenced_by=None,
@@ -403,7 +419,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="number",
                 value="4",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 6),
                 referenced_by=None,
@@ -411,7 +427,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="variable_value_end",
                 value="\n",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 7),
                 referenced_by=None,
@@ -419,7 +435,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="variable_name",
                 value="$y:",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 0),
                 referenced_by=None,
@@ -427,7 +443,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 3),
                 referenced_by=None,
@@ -435,7 +451,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="number",
                 value="6",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 4),
                 referenced_by=None,
@@ -443,7 +459,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 5),
                 referenced_by=None,
@@ -451,7 +467,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="number",
                 value="2",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 4),
                 referenced_by=ReferencedBy(
@@ -461,7 +477,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 5),
                 referenced_by=ReferencedBy(
@@ -471,7 +487,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="number",
                 value="4",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 6),
                 referenced_by=ReferencedBy(
@@ -481,7 +497,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 8),
                 referenced_by=None,
@@ -489,7 +505,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="number",
                 value="2",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 9),
                 referenced_by=None,
@@ -497,7 +513,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="variable_value_end",
                 value="\n",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 10),
                 referenced_by=None,
@@ -505,7 +521,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="selector_start_class",
                 value=".thing",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(2, 0),
                 referenced_by=None,
@@ -513,7 +529,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(2, 6),
                 referenced_by=None,
@@ -521,7 +537,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="declaration_set_start",
                 value="{",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(2, 7),
                 referenced_by=None,
@@ -529,7 +545,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(2, 8),
                 referenced_by=None,
@@ -537,7 +553,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="declaration_name",
                 value="border:",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(2, 9),
                 referenced_by=None,
@@ -545,7 +561,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(2, 16),
                 referenced_by=None,
@@ -553,7 +569,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="number",
                 value="6",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 4),
                 referenced_by=ReferencedBy(
@@ -563,7 +579,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 5),
                 referenced_by=ReferencedBy(
@@ -573,7 +589,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="number",
                 value="2",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 4),
                 referenced_by=ReferencedBy(
@@ -583,7 +599,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 5),
                 referenced_by=ReferencedBy(
@@ -593,7 +609,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="number",
                 value="4",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 6),
                 referenced_by=ReferencedBy(
@@ -603,7 +619,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 8),
                 referenced_by=ReferencedBy(
@@ -613,7 +629,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="number",
                 value="2",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 9),
                 referenced_by=ReferencedBy(
@@ -623,7 +639,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(2, 19),
                 referenced_by=None,
@@ -631,7 +647,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="declaration_set_end",
                 value="}",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(2, 20),
                 referenced_by=None,
@@ -640,11 +656,11 @@ class TestVariableReferenceSubstitution:
 
     def test_variable_used_inside_property_value(self):
         css = "$x: red\n.thing { border: on $x; }"
-        assert list(substitute_references(tokenize(css, ""))) == [
+        assert list(substitute_references(tokenize(css, ("", "")))) == [
             Token(
                 name="variable_name",
                 value="$x:",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 0),
                 referenced_by=None,
@@ -652,7 +668,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 3),
                 referenced_by=None,
@@ -660,7 +676,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="token",
                 value="red",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 4),
                 referenced_by=None,
@@ -668,7 +684,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="variable_value_end",
                 value="\n",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 7),
                 referenced_by=None,
@@ -676,7 +692,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="selector_start_class",
                 value=".thing",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 0),
                 referenced_by=None,
@@ -684,7 +700,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 6),
                 referenced_by=None,
@@ -692,7 +708,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="declaration_set_start",
                 value="{",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 7),
                 referenced_by=None,
@@ -700,7 +716,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 8),
                 referenced_by=None,
@@ -708,7 +724,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="declaration_name",
                 value="border:",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 9),
                 referenced_by=None,
@@ -716,7 +732,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 16),
                 referenced_by=None,
@@ -724,7 +740,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="token",
                 value="on",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 17),
                 referenced_by=None,
@@ -732,7 +748,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 19),
                 referenced_by=None,
@@ -740,7 +756,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="token",
                 value="red",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 4),
                 referenced_by=ReferencedBy(
@@ -750,7 +766,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="declaration_end",
                 value=";",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 22),
                 referenced_by=None,
@@ -758,7 +774,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 23),
                 referenced_by=None,
@@ -766,7 +782,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="declaration_set_end",
                 value="}",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(1, 24),
                 referenced_by=None,
@@ -775,11 +791,11 @@ class TestVariableReferenceSubstitution:
 
     def test_variable_definition_eof(self):
         css = "$x: 1"
-        assert list(substitute_references(tokenize(css, ""))) == [
+        assert list(substitute_references(tokenize(css, ("", "")))) == [
             Token(
                 name="variable_name",
                 value="$x:",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 0),
                 referenced_by=None,
@@ -787,7 +803,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 3),
                 referenced_by=None,
@@ -795,7 +811,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="number",
                 value="1",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 4),
                 referenced_by=None,
@@ -804,11 +820,11 @@ class TestVariableReferenceSubstitution:
 
     def test_variable_reference_whitespace_trimming(self):
         css = "$x:    123;.thing{border: $x}"
-        assert list(substitute_references(tokenize(css, ""))) == [
+        assert list(substitute_references(tokenize(css, ("", "")))) == [
             Token(
                 name="variable_name",
                 value="$x:",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 0),
                 referenced_by=None,
@@ -816,7 +832,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value="    ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 3),
                 referenced_by=None,
@@ -824,7 +840,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="number",
                 value="123",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 7),
                 referenced_by=None,
@@ -832,7 +848,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="variable_value_end",
                 value=";",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 10),
                 referenced_by=None,
@@ -840,7 +856,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="selector_start_class",
                 value=".thing",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 11),
                 referenced_by=None,
@@ -848,7 +864,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="declaration_set_start",
                 value="{",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 17),
                 referenced_by=None,
@@ -856,7 +872,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="declaration_name",
                 value="border:",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 18),
                 referenced_by=None,
@@ -864,7 +880,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="whitespace",
                 value=" ",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 25),
                 referenced_by=None,
@@ -872,7 +888,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="number",
                 value="123",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 7),
                 referenced_by=ReferencedBy(
@@ -882,7 +898,7 @@ class TestVariableReferenceSubstitution:
             Token(
                 name="declaration_set_end",
                 value="}",
-                path="",
+                read_from=("", ""),
                 code=css,
                 location=(0, 28),
                 referenced_by=None,
@@ -1184,12 +1200,6 @@ class TestParseTextAlign:
         rules = stylesheet._parse_rules(css, "foo")
         assert rules[0].errors
 
-    def test_text_align_empty_uses_default(self):
-        css = "#foo { text-align: ; }"
-        stylesheet = Stylesheet()
-        stylesheet.add_source(css)
-        assert stylesheet.rules[0].styles.text_align == "start"
-
 
 class TestTypeNames:
     def test_type_no_number(self):
@@ -1228,7 +1238,7 @@ class TestTypeNames:
                 stylesheet.parse()
 
 
-def test_parse_bad_psuedo_selector():
+def test_parse_bad_pseudo_selector():
     """Check unknown selector raises a token error."""
 
     bad_selector = """\
@@ -1238,9 +1248,51 @@ Widget:foo{
     """
 
     stylesheet = Stylesheet()
-    stylesheet.add_source(bad_selector, "foo")
+    stylesheet.add_source(bad_selector, None)
 
     with pytest.raises(TokenError) as error:
         stylesheet.parse()
 
-    assert error.value.start == (0, 6)
+    assert error.value.start == (1, 7)
+
+
+def test_parse_bad_pseudo_selector_with_suggestion():
+    """Check unknown pseudo selector raises token error with correct position."""
+
+    bad_selector = """
+Widget:blu {
+    border: red;
+}
+"""
+
+    stylesheet = Stylesheet()
+    stylesheet.add_source(bad_selector, None)
+
+    with pytest.raises(TokenError) as error:
+        stylesheet.parse()
+
+    assert error.value.start == (2, 7)
+
+
+@pytest.mark.parametrize(
+    "selector, expected",
+    [
+        # True cases
+        ("#foo", True),
+        ("#bar", True),
+        ("#f", True),
+        # False cases
+        ("#", False),
+        ("Foo", False),
+        (".foo", False),
+        ("#5foo", False),
+        ("#foo .bar", False),
+        ("#foo>.bar", False),
+        ("#foo.bar", False),
+        (".foo #foo", False),
+        ("#foo #bar", False),
+    ],
+)
+def test_is_id_selector(selector: str, expected: bool) -> None:
+    """Test is_id_selector is working as expected."""
+    assert is_id_selector(selector) is expected
