@@ -4,7 +4,6 @@ from textual.widgets import Checkbox, Footer
 
 class ScrollOffByOne(App):
     AUTO_FOCUS = None
-    HOVER_EFFECTS_SCROLL_PAUSE = 0.0
 
     def compose(self) -> ComposeResult:
         for number in range(1, 100):
@@ -12,7 +11,7 @@ class ScrollOffByOne(App):
         yield Footer()
 
     def on_mount(self) -> None:
-        self.query_one("Screen").scroll_end()
+        self.screen.scroll_end()
 
 
 app = ScrollOffByOne()
