@@ -5,7 +5,7 @@ from textual.containers import Container
 from textual.widgets import Label
 
 
-CSS_PATH = (Path(__file__) / "../hot_reloading_app.css").resolve()
+CSS_PATH = (Path(__file__) / "../hot_reloading_app.tcss").resolve()
 
 # Write some CSS to the file before the app loads.
 # Then, the test will clear all the CSS to see if the
