@@ -1,6 +1,6 @@
 from textual.app import App, ComposeResult
 from textual.containers import VerticalScroll
-from textual.widgets import Label
+from textual.widgets import Label, Static
 
 
 class LoadingApp(App[None]):
@@ -25,12 +25,31 @@ async def test_loading_disables_and_remove_scrollbars():
     async with app.run_test() as pilot:
         vs = app.query_one(VerticalScroll)
         # Sanity checks:
-        assert not vs.disabled
-        assert vs.styles.overflow_y != "hidden"
+        assert not vs._check_disabled()
 
         await pilot.press("l")
         await pilot.pause()
 
-        assert vs.disabled
-        assert vs.styles.overflow_x == "hidden"
-        assert vs.styles.overflow_y == "hidden"
+        assert vs._check_disabled()
+
+
+async def test_premature_loading():
+    """Test a widget can set the `loading` attribute before mounting."""
+
+    # No assert, we're just expecting it to not crash
+
+    class LoadingWidget(Static):
+        """A widget that shows a loading indicator."""
+
+    class LoadingApp(App[None]):
+        """Simple app with a single loading widget."""
+
+        def compose(self) -> ComposeResult:
+            """Compose the app with the loading widget."""
+            widget = LoadingWidget("Loading content...")
+            widget.loading = True  # Should not crash
+            yield widget
+
+    app = LoadingApp()
+    async with app.run_test() as pilot:
+        await pilot.pause()
