@@ -1,11 +1,14 @@
+import threading
+
 import pytest
 
+from textual._dispatch_key import dispatch_key
 from textual.app import App, ComposeResult
 from textual.errors import DuplicateKeyHandlers
 from textual.events import Key
 from textual.message import Message
 from textual.widget import Widget
-from textual.widgets import Input
+from textual.widgets import Button, Input, Label
 
 
 class ValidWidget(Widget):
@@ -20,7 +23,7 @@ class ValidWidget(Widget):
 
 async def test_dispatch_key_valid_key():
     widget = ValidWidget()
-    result = await widget.dispatch_key(Key(key="x", character="x"))
+    result = await dispatch_key(widget, Key(key="x", character="x"))
     assert result is True
     assert widget.called_by == widget.key_x
 
@@ -29,7 +32,7 @@ async def test_dispatch_key_valid_key_alias():
     """When you press tab or ctrl+i, it comes through as a tab key event, but handlers for
     tab and ctrl+i are both considered valid."""
     widget = ValidWidget()
-    result = await widget.dispatch_key(Key(key="tab", character="\t"))
+    result = await dispatch_key(widget, Key(key="tab", character="\t"))
     assert result is True
     assert widget.called_by == widget.key_ctrl_i
 
@@ -55,7 +58,7 @@ async def test_dispatch_key_raises_when_conflicting_handler_aliases():
     In the terminal, they're the same thing, so we fail fast via exception here."""
     widget = DuplicateHandlersWidget()
     with pytest.raises(DuplicateKeyHandlers):
-        await widget.dispatch_key(Key(key="tab", character="\t"))
+        await dispatch_key(widget, Key(key="tab", character="\t"))
     assert widget.called_by == widget.key_tab
 
 
@@ -143,3 +146,47 @@ async def test_prevent_with_call_next() -> None:
         app.call_next(app.change_input)
         await pilot.pause()
         assert hits == 2
+
+
+async def test_prevent_default():
+    """Test that prevent_default doesn't apply when a message is bubbled."""
+
+    app_button_pressed = False
+
+    class MyButton(Button):
+        def _on_button_pressed(self, event: Button.Pressed) -> None:
+            event.prevent_default()
+
+    class PreventApp(App[None]):
+        def compose(self) -> ComposeResult:
+            yield MyButton("Press me")
+            yield Label("No pressure")
+
+        def on_button_pressed(self, event: Button.Pressed) -> None:
+            nonlocal app_button_pressed
+            app_button_pressed = True
+            self.query_one(Label).update("Ouch!")
+
+    app = PreventApp()
+    async with app.run_test() as pilot:
+        await pilot.click(MyButton)
+        assert app_button_pressed
+
+
+async def test_thread_safe_post_message():
+    class TextMessage(Message):
+        pass
+
+    class TestApp(App):
+
+        def on_mount(self) -> None:
+            msg = TextMessage()
+            threading.Thread(target=self.post_message, args=(msg,)).start()
+
+        def on_text_message(self, message):
+            self.exit()
+
+    app = TestApp()
+
+    async with app.run_test() as pilot:
+        await pilot.pause()
