@@ -1,8 +1,6 @@
 from __future__ import annotations
 
-from rich.console import Console
 from rich.segment import Segment
-from rich.style import Style
 
 from textual._styles_cache import StylesCache
 from textual.color import Color
@@ -26,36 +24,6 @@ def test_set_dirty():
     assert not cache.is_dirty(6)
 
 
-def test_no_styles():
-    """Test that empty style returns the content un-altered"""
-    content = [
-        Strip([Segment("foo")]),
-        Strip([Segment("bar")]),
-        Strip([Segment("baz")]),
-    ]
-    styles = Styles()
-    cache = StylesCache()
-    lines = cache.render(
-        styles,
-        Size(3, 3),
-        Color.parse("blue"),
-        Color.parse("green"),
-        content.__getitem__,
-        Console(),
-        "",
-        "",
-        content_size=Size(3, 3),
-    )
-    style = Style.from_color(bgcolor=Color.parse("green").rich_color)
-    expected = [
-        Strip([Segment("foo", style)], 3),
-        Strip([Segment("bar", style)], 3),
-        Strip([Segment("baz", style)], 3),
-    ]
-
-    assert lines == expected
-
-
 def test_border():
     content = [
         Strip([Segment("foo")]),
@@ -71,7 +39,7 @@ def test_border():
         Color.parse("blue"),
         Color.parse("green"),
         content.__getitem__,
-        Console(),
+        [],
         None,
         None,
         content_size=Size(3, 3),
@@ -105,7 +73,7 @@ def test_padding():
         Color.parse("blue"),
         Color.parse("green"),
         content.__getitem__,
-        Console(),
+        [],
         None,
         None,
         content_size=Size(3, 3),
@@ -140,7 +108,7 @@ def test_padding_border():
         Color.parse("blue"),
         Color.parse("green"),
         content.__getitem__,
-        Console(),
+        [],
         None,
         None,
         content_size=Size(3, 3),
@@ -176,7 +144,7 @@ def test_outline():
         Color.parse("blue"),
         Color.parse("green"),
         content.__getitem__,
-        Console(),
+        [],
         None,
         None,
         content_size=Size(3, 3),
@@ -207,7 +175,7 @@ def test_crop():
         Color.parse("blue"),
         Color.parse("green"),
         content.__getitem__,
-        Console(),
+        [],
         None,
         None,
         content_size=Size(3, 3),
@@ -246,7 +214,7 @@ def test_dirty_cache() -> None:
         Color.parse("blue"),
         Color.parse("green"),
         get_content_line,
-        Console(),
+        [],
         None,
         None,
         content_size=Size(3, 3),
@@ -274,7 +242,7 @@ def test_dirty_cache() -> None:
         Color.parse("blue"),
         Color.parse("green"),
         get_content_line,
-        Console(),
+        [],
         None,
         None,
         content_size=Size(3, 3),
@@ -293,7 +261,7 @@ def test_dirty_cache() -> None:
         Color.parse("blue"),
         Color.parse("green"),
         get_content_line,
-        Console(),
+        [],
         None,
         None,
         content_size=Size(3, 3),
